@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,14 +25,14 @@ from qt import QGraphicsRectItem, QGraphicsSimpleTextItem
 from qt import QPushButton, QMessageBox, QComboBox
 
 
-from common import Internal, isAlive, unicode, Debug
+from common import Internal, isAlive, Debug
 from wind import Wind
 from tilesource import TileSource
 from animation import animate
-from log import logError, logDebug, logWarning, m18n
+from log import logError, logDebug, logWarning, i18n
 from query import Query
 from uitile import UITile
-from board import WindLabel, Board, rotateCenter
+from board import WindLabel, Board
 from game import Game
 from games import Games
 from hand import Hand
@@ -40,8 +40,8 @@ from handboard import HandBoard, TileAttr
 from player import Player, Players
 from visible import VisiblePlayer
 from tables import SelectRuleset
-from uiwall import UIWall
-from guiutil import decorateWindow, BlockSignals
+from uiwall import UIWall, SideText
+from guiutil import decorateWindow, BlockSignals, rotateCenter, sceneRotation
 
 
 class SwapDialog(QMessageBox):
@@ -50,13 +50,13 @@ class SwapDialog(QMessageBox):
 
     def __init__(self, swappers):
         QMessageBox.__init__(self)
-        decorateWindow(self, m18n("Swap Seats"))
+        decorateWindow(self, i18nc("@title:window", "Swap Seats"))
         self.setText(
-            m18n("By the rules, %1 and %2 should now exchange their seats. ",
+            i18n("By the rules, %1 and %2 should now exchange their seats. ",
                  swappers[0].name, swappers[1].name))
-        self.yesAnswer = QPushButton(m18n("&Exchange"))
+        self.yesAnswer = QPushButton(i18n("&Exchange"))
         self.addButton(self.yesAnswer, QMessageBox.YesRole)
-        self.noAnswer = QPushButton(m18n("&Keep seat"))
+        self.noAnswer = QPushButton(i18n("&Keep seat"))
         self.addButton(self.noAnswer, QMessageBox.NoRole)
 
 
@@ -67,7 +67,7 @@ class SelectPlayers(SelectRuleset):
     def __init__(self):
         SelectRuleset.__init__(self)
         Players.load()
-        decorateWindow(self, m18n('Select four players'))
+        decorateWindow(self, i18nc("@title:window", "Select four players"))
         self.names = None
         self.nameWidgets = []
         for idx, wind in enumerate(Wind.all4):
@@ -92,7 +92,7 @@ class SelectPlayers(SelectRuleset):
                         if playerIdx >= 0:
                             cbName.setCurrentIndex(playerIdx)
                     except KeyError:
-                        logError(u'database is inconsistent: player with id %d is in game but not in player'
+                        logError('database is inconsistent: player with id %d is in game but not in player'
                                  % playerId)
         self.slotValidate()
 
@@ -102,7 +102,7 @@ class SelectPlayers(SelectRuleset):
 
     def __selectedNames(self):
         """A set with the currently selected names"""
-        return set(list(unicode(cbName.currentText()) for cbName in self.nameWidgets))
+        return set(list(cbName.currentText() for cbName in self.nameWidgets))
 
     def slotValidate(self):
         """try to find 4 different players and update status of the Ok button"""
@@ -113,17 +113,17 @@ class SelectPlayers(SelectRuleset):
         allNames = set(Players.humanNames.values())
         unusedNames = allNames - self.__selectedNames()
         with BlockSignals(self.nameWidgets):
-            used = set([unicode(x.currentText())
+            used = set([x.currentText()
                         for x in self.nameWidgets if x.manualSelect])
             for combo in self.nameWidgets:
                 if not combo.manualSelect:
-                    if unicode(combo.currentText()) in used:
+                    if combo.currentText() in used:
                         comboName = unusedNames.pop()
                         combo.clear()
                         combo.addItems([comboName])
-                        used.add(unicode(combo.currentText()))
+                        used.add(combo.currentText())
             for combo in self.nameWidgets:
-                comboName = unicode(combo.currentText())
+                comboName = combo.currentText()
                 combo.clear()
                 combo.addItems([comboName])
                 combo.addItems(sorted(
@@ -131,7 +131,7 @@ class SelectPlayers(SelectRuleset):
                 combo.setCurrentIndex(0)
         self.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
             len(self.__selectedNames()) == 4)
-        self.names = list(unicode(cbName.currentText())
+        self.names = list(cbName.currentText()
                           for cbName in self.nameWidgets)
 
 
@@ -195,7 +195,7 @@ class ScoringHandBoard(HandBoard):
 
     def sync(self, adding=None):  # pylint: disable=unused-argument
         """place all tiles in ScoringHandBoard"""
-        self.calcPlaces(sum(self.uiMelds, []))
+        self.placeTiles(sum(self.uiMelds, []))
 
     def deselect(self, meld):
         """remove meld from old board"""
@@ -237,21 +237,22 @@ class ScoringHandBoard(HandBoard):
         self._noPen()
 
     def dropTile(self, uiTile, lowerHalf):
-        """drop meld or uiTile into lower or upper half of our hand"""
+        """drop uiTile into lower or upper half of our hand"""
         senderBoard = uiTile.board
-        self.checkTiles()
-        senderBoard.checkTiles()
         newMeld = senderBoard.chooseVariant(uiTile, lowerHalf)
         if not newMeld:
-            self.checkTiles()
-            senderBoard.checkTiles()
             return False
         uiMeld = senderBoard.assignUITiles(uiTile, newMeld)
-        senderBoard.deselect(uiMeld)
         for uiTile, tile in zip(uiMeld, newMeld):
             uiTile.tile = tile
+        self.dropMeld(uiMeld)
+
+    def dropMeld(self, uiMeld):
+        """drop uiMeld into our hand"""
+        senderBoard = uiMeld[0].board
+        senderBoard.deselect(uiMeld)
         self.uiMelds.append(uiMeld)
-        self.player.addMeld(newMeld)
+        self.player.addMeld(uiMeld.meld)
         self.sync()
         self.hasFocus = senderBoard == self or not senderBoard.uiTiles
         self.checkTiles()
@@ -297,8 +298,8 @@ class ScoringHandBoard(HandBoard):
                     hbCenter.x() * 1,
                     1)
                 helpItems = [splitter]
-                for name, yFactor in [(m18n('Move Exposed Tiles Here'), 0.5),
-                                      (m18n('Move Concealed Tiles Here'), 1.5)]:
+                for name, yFactor in [(i18n('Move Exposed Tiles Here'), 0.5),
+                                      (i18n('Move Concealed Tiles Here'), 1.5)]:
                     helper = QGraphicsSimpleTextItem(name, self)
                     helper.setScale(3)
                     nameRect = QRectF()
@@ -307,7 +308,7 @@ class ScoringHandBoard(HandBoard):
                     center = QPointF(hbCenter)
                     center.setY(center.y() * yFactor)
                     helper.setPos(center - nameRect.center())
-                    if self.sceneRotation() == 180:
+                    if sceneRotation(self) == 180:
                         rotateCenter(helper, 180)
                     helpItems.append(helper)
                 self.__moveHelper = self.scene().createItemGroup(helpItems)
@@ -339,6 +340,8 @@ class ScoringPlayer(VisiblePlayer, Player):
         if self.game and self.game.wall:
             # is None while __del__
             self.front = self.game.wall[self.idx]
+            if hasattr(self, 'sideText'):
+                self.sideText.board = self.front
         if isAlive(self.handBoard):
             self.handBoard.setEnabled(True)
             self.handBoard.showMoveHelper()
@@ -475,10 +478,10 @@ class ScoringPlayer(VisiblePlayer, Player):
                         break
             if not popped:
                 logDebug(
-                    u'%s: %s.removeMeld did not find %s' %
+                    '%s: %s.removeMeld did not find %s' %
                     (self.name, self.__class__.__name__, meld), showStack=3)
-                logDebug(u'    concealed: %s' % self._concealedMelds)
-                logDebug(u'      exposed: %s' % self._exposedMelds)
+                logDebug('    concealed: %s' % self._concealedMelds)
+                logDebug('      exposed: %s' % self._exposedMelds)
             else:
                 if Debug.scoring:
                     logDebug('{} lost meld {}'.format(self, popped))
@@ -506,9 +509,9 @@ class ScoringGame(Game):
         scene.selectorBoard.load(self)
         self.prepareHand()
         self.initHand()
-        Internal.scene.mainWindow.adjustView()
+        Internal.scene.mainWindow.adjustMainView()
         Internal.scene.mainWindow.updateGUI()
-        self.wall.decorate()
+        self.wall.decorate4()
         self.throwDices()
 
     @Game.seed.getter
@@ -555,6 +558,7 @@ class ScoringGame(Game):
             player.hide()
         if self.wall:
             self.wall.hide()
+        SideText.removeAll()
         return Game.close(self)
 
     @staticmethod
@@ -606,7 +610,7 @@ def scoreGame():
     Players.load()
     if len(Players.humanNames) < 4:
         logWarning(
-            m18n('Please define four players in <interface>Settings|Players</interface>'))
+            i18n('Please define four players in <interface>Settings|Players</interface>'))
         return
     gameSelector = Games(Internal.mainWindow)
     selected = None
